/*-
 * Copyright (c) 2002 Doug White
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * 
 */

/*
 *  get-lan-config.c
 *
 */
#include <stdlib.h>
#include <stdio.h>
#include "kcs.h"
#include "kcs-common.h"
#include "ipmi-common.h"

#define KCSGET_REQUEST_LEN	3
#define KCSGET_RESPONSE_LEN	0xff

int
main(int argc, char** argv)
{
#ifdef DEBUG
    int u;
#endif
#ifdef DEBUGOUTPUT
    int u;
#endif
    int t;
    int cnum=0xe;
    char cmd[KCSGET_REQUEST_LEN];
    char out[KCSGET_RESPONSE_LEN];
    bzero(cmd, KCSGET_REQUEST_LEN);
    bzero(out, 24);
    /* Get permission to use I/O portrange */

    if(argc ==2) {
       cnum = atoi(argv[1]);
    } 

    t = i386_set_ioperm(KCS_BASE_IOPORT, 0x2, 0x1);
    if(t != 0) {
        perror("i386_set_ioperm");
        exit(1);
    }
    /* IPMI Get Channel Info
     * NetFn App (0x6), Command 0x42, LUN 0x0
     */
    /* There's a lot of these, selected by param/block selectors.
     * not easy to iterate over either. */

    /* KCS Command Bytes */
    cmd[0] = ipmi_compose_netfn_lun(0x6, 0x0);
    cmd[1] = 0x42;  /* Get Channel Info */

    /* Request Data Bytes */
    cmd[2] = cnum;  /* channel number */
    
    /* Write out the command. */
    if(kcs_write(cmd, KCSGET_REQUEST_LEN) == -1)
    {
        printf("Error writing cmd\n");
        exit(1);
    }
    
    /* read back the response. */
    if(kcs_read(out, sizeof(out)) == -1)
    {
        printf("Read failed\n");
        exit(2);
    }
    
    /* Decode output. */
    printf("LUN/NetFn      : 0x%x/0x%x\n", out[0] & 0x3, out[0] >> 2);	
    printf("Command        : 0x%x\n", out[1]);
    printf("Completion Code: 0x%x\n", (out[2] & 0xff));
    printf("Channel ID     : 0x%x\n", out[3]);
    printf("Channel Medium : 0x%x\n", out[4]);
    switch(out[4] & 0x7f) {
    case 0x1: printf("               : IPMB (I2C)\n"); break;
    case 0x2: printf("               : ICMBv1.0\n"); break;
    case 0x3: printf("               : ICMBv0.9\n"); break;
    case 0x4: printf("               : 802.3 LAN\n"); break;
    case 0x5: printf("               : RS-232\n"); break;
    case 0x6: printf("               : Other LAN\n"); break;
    case 0x7: printf("               : PCI SMBus\n"); break;
    case 0x8: printf("               : SMBus v1.0/v1.1\n"); break;
    case 0x9: printf("               : SMBus v2.0\n"); break;
    case 0xa: printf("               : USB v1.X\n"); break;
    case 0xb: printf("               : USB v2.X\n"); break;
    case 0xc: printf("               : System Interface\n"); break;
    }
    printf("Chan Protocol  : 0x%x\n", out[5]);
    switch((out[5] & 0x1f)) {
    case 0x1: printf("               : IPMB, RS-232, LAN\n"); break;
    case 0x2: printf("               : ICMB v1.0\n"); break;
    case 0x4: printf("               : IPMI-SMBus\n"); break;
    case 0x5: printf("               : KCS\n"); break;
    case 0x6: printf("               : SMIC\n"); break;
    case 0x7: printf("               : BulkTransfer v1.0\n"); break;
    case 0x8: printf("               : BulkTransfer v1.5\n"); break;
    case 0x9: printf("               : Terminal Mode\n"); break;
    case 0x1c: printf("               : OEM 1\n"); break;
    case 0x1d: printf("               : OEM 2\n"); break;
    case 0x1e: printf("               : OEM 3\n"); break;
    case 0x1f: printf("               : OEM 4\n"); break;
    }
    printf("Session Support: 0x%x\n", (out[6] & 0xb0));
    switch(out[6] & 0xb0) {
    case 0x0: printf("               : Sessionless\n"); break;
    case 0x1: printf("               : Single-session\n"); break;
    case 0x2: printf("               : Multisession\n"); break;
    case 0x4: printf("               : Session-based (single or multiple)\n"); break;
    }
    printf("Mfr ID         : 0x%x%x%x\n", out[8], out[7], out[6]);
    printf("Auxiliary Info : 0x%x%x\n", out[10], out[9]);

#ifdef DEBUGOUTPUT
    for(u=0; u<24; u++) {
            if(u%8 == 0) printf("\n");
            printf("0x%x ", out[u]);
    }
#endif
    printf("\n");
}

    
     
