/*-
 * Copyright (c) 2002 Doug White
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * 
 */

#include <sys/types.h>
#include <machine/cpufunc.h>
#include <machine/sysarch.h>
#include <stdio.h>
#include "kcs.h"
#include "kcs-common.h"

int
main(int argc, char** argv)
{
	int t;

	t = i386_set_ioperm(KCS_BASE_IOPORT, 0x2, 0x1);
	if(t != 0) {
		perror("i386_set_ioperm");
		exit(1);
	}

        printf("START: Wait for IBF clear\n");
        kcs_wait_for_ibf_clear();
        printf("1: Send ABORT command\n");
	kcs_abort();
        printf("2: Wait for IBF clear\n");
	kcs_wait_for_ibf_clear();
        printf("3: Clear OBF if necessary\n");
        kcs_clear_obf();	
        printf("4: Write NULL interrupt byte\n");
	kcs_write_byte(0x00);
	printf("5: Wait for IBF clear\n");
        kcs_wait_for_ibf_clear();
        printf("6: Check if in READ state\n");
        if(!kcs_test_if_state(KCS_READ))
        {
            printf("ERROR: Not in read state\n");
            exit(1);
        }
        printf("7: Wait for OBF set\n");
	kcs_wait_for_obf_set();
        printf("8: Read status code\n");
	printf(" : Status Code 0x%x\n", kcs_read_byte());
        printf("9: Write Read Next Cmd to Ack Status Code\n");
        kcs_read_next();
        printf("10: Wait for IBF clear\n");
        kcs_wait_for_ibf_clear();
        printf("11: Verify IDLE state\n");
        if(!kcs_test_if_state(KCS_IDLE))
        {
            printf("ERROR: not in Idle, reset failed\n");
            exit(1);
        }
        printf("12: Wait for OBF set\n");
        kcs_wait_for_obf_set();
        printf("13: Clear OBF\n");
        kcs_clear_obf();
 	
	printf("Reset successful.\n");	
	
}

