/*-
 * Copyright (c) 2002 Doug White
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * 
 */

/*
 * kcs-decode.c
 * 
 */

#include <unistd.h>
#include <stdio.h>
#include "kcs.h"
#include "kcs-common.h"
#include "ipmi-common.h"

/* Take data and datalen and try to figure out if we can
 * do something with it.
 */
int
kcs_decode_packet(int len, u_int8_t* data) {

	u_int8_t netfn, lun, cmd;
	/* Rip off the netfn, lun, and command for lookup */
	netfn = data[0] >> 2;
	lun = data[0] & 0x3;
	cmd = data[1];

	/* The (netfn, cmd) tuple uniquely identifies the _response_
	 * to a command. Responses use the next NetFn up.
	 */

	/* Application functions */
	switch(netfn-1) {
	case IPMI_NETFN_APP:
		switch(cmd) {
		case IPMI_CMD_GET_DEVICE_ID:
			printf("Device ID      : %#0.2x\n", data[3]);
			printf("Device Revision: %#0.2x\n", data[4]);
			printf("Firmware Rev 1 : %#0.2x\n", data[5]);
			printf("Firmware Rev 2 : %#0.2x\n", data[6]);
			printf("IPMI Version   : %#0.2x\n", data[7]);
			printf("Adt'l Dev Supt : %#0.2x\n", data[8]);
			if(data[8] & 0x1) 
				printf("               : Sensor Device\n");
			if(data[8] & 0x2) 
				printf("               : Sensor Data Record (SDR) Repository\n");
			if(data[8] & 0x4) 
				printf("               : System Event Log (SEL)\n");
			if(data[8] & 0x8) 
				printf("               : Field-Replaceable Unit (FRU) Inventory\n");
			if(data[8] & 0x10) 
				printf("               : IPMB Event Receiver\n");
			if(data[8] & 0x20) 
				printf("               : IPMB Event Generator\n");
			if(data[8] & 0x40) 
				printf("               : IPMI Message Bridging\n");
			if(data[8] & 0x80) 
				printf("               : Chassis Device\n");
			/* printf("Mfr ID         : %#x%x%x\n", data[11], data[10], data[9]); 
			printf("Product ID     : %#x%x\n", data[13], data[12]);
			printf("Aux FW Info    : %#x%x%x\n", data[16], data[15], data[14]); */
			printf("Mfr ID         : %#x\n", (data[9] + (data[10] << 8) + (data[11] << 8)));
			printf("Product ID     : %#x\n", ((data[13]<<8) + data[12]));
			printf("Aux FW Info    : %#x\n", ((data[16]<<16) + (data[15]<<8) + data[14]));
			break;
		case IPMI_CMD_GET_CHANNEL_INFO:
			    printf("Channel ID     : %#0.2x\n", data[3]);
			    printf("Channel Medium : %#0.2x\n", data[4]);
			    switch(data[4] & 0x7f) {
			    case 0x1: printf("               : IPMB (I2C)\n"); break;
			    case 0x2: printf("               : ICMBv1.0\n"); break;
			    case 0x3: printf("               : ICMBv0.9\n"); break;
			    case 0x4: printf("               : 802.3 LAN\n"); break;
			    case 0x5: printf("               : RS-232\n"); break;
			    case 0x6: printf("               : Other LAN\n"); break;
			    case 0x7: printf("               : PCI SMBus\n"); break;
			    case 0x8: printf("               : SMBus v1.0/v1.1\n"); break;
			    case 0x9: printf("               : SMBus v2.0\n"); break;
			    case 0xa: printf("               : USB v1.X\n"); break;
			    case 0xb: printf("               : USB v2.X\n"); break;
			    case 0xc: printf("               : System Interface\n"); break;
			    }
			    printf("Chan Protocol  : %#0.2x\n", data[5]);
			    switch((data[5] & 0x1f)) {
			    case 0x1: printf("               : IPMB, RS-232, LAN\n"); break;
			    case 0x2: printf("               : ICMB v1.0\n"); break;
			    case 0x4: printf("               : IPMI-SMBus\n"); break;
			    case 0x5: printf("               : KCS\n"); break;
			    case 0x6: printf("               : SMIC\n"); break;
			    case 0x7: printf("               : BulkTransfer v1.0\n"); break;
			    case 0x8: printf("               : BulkTransfer v1.5\n"); break;
			    case 0x9: printf("               : Terminal Mode\n"); break;
			    case 0x1c: printf("               : OEM 1\n"); break;
			    case 0x1d: printf("               : OEM 2\n"); break;
			    case 0x1e: printf("               : OEM 3\n"); break;
			    case 0x1f: printf("               : OEM 4\n"); break;
			    }
			    printf("Session Support: %#0.2x\n", (data[6] & 0xb0));
			    switch(data[6] & 0xb0) {
			    case 0x0: printf("               : Sessionless\n"); break;
			    case 0x1: printf("               : Single-session\n"); break;
			    case 0x2: printf("               : Multisession\n"); break;
			    case 0x4: printf("               : Session-based (single or multiple)\n"); break;
			    }
			    printf("Mfr ID         : %#x%x%x\n", data[8], data[7], data[6]);
			    printf("Auxiliary Info : %#x%x\n", data[10], data[9]);
			    break;
		} /* app switch */
		break;

	case IPMI_NETFN_TRANSPORT:
		switch(cmd) {
		case IPMI_CMD_GET_LAN_CONFIG:
			printf("Parameter Rev  : %#x\n", data[3]);
			break;
		} /* transport switch */
	} /* netfn switch */
	
	return 0;
}
