/*-
 * Copyright (c) 2002 Doug White
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * 
 */

#include <sys/types.h>
#include <machine/cpufunc.h>
#include <machine/sysarch.h>
#include <stdio.h>
#include "kcs.h"

int
main(int argc, char** argv)
{
	int t, u, v;
	int status = 0;
	char a[3];
	char out[24];

	t = i386_set_ioperm(KCS_BASE_IOPORT, 0x2, 0x1);
	if(t != 0) {
		perror("i386_set_ioperm");
		exit(1);
	}

	status = 0;
	status = inb(KCS_STATUS_IOPORT);
	printf("Current KCS state: 0x%x : ", status);
	if((status & KCS_STATBITS) == KCS_IDLE) {
		printf("IDLE_STATE ");
	} else if((status & KCS_STATBITS) == KCS_READ) {
		printf("READ_STATE ");
	} else if((status & KCS_STATBITS) == KCS_WRITE) {
		printf("WRITE_STATE ");
	} else if((status & KCS_STATBITS) == KCS_ERROR) {
		printf("ERROR_STATE ");
	} else {
		printf("Huh? ");
	}
	if(status & KCS_IBF) {
		printf("IBF ");
	}
	if(status & KCS_OBF) {
		printf("OBF ");
	}
	if(status & KCS_OEM1) {
		printf("OEM1 ");
	}
	if(status & KCS_OEM2) {
		printf("OEM2 ");
	}
	printf("\n");
	/* printf("Current KCS statebits: 0x%x\n", status & KCS_STATBITS); */

	/* Compose our IPMI Get Device ID command */
	/* GDID is NetFn App, CMD 0x01, and are directed to LUN 0. */
	/* NetFn App Req = 0x6, Resp = 0x7
	/* First word is LLAAAAAA */
	a[0] = 0x6 << 2;
	a[1] = 0x01;
	a[2] = 0x00;

	status = inb(KCS_STATUS_IOPORT);
	while((status & KCS_IBF)) {
		printf("Waiting for IBF to clear to start\n");
		sleep(1);
		status = inb(KCS_STATUS_IOPORT);
	}
	/* Start up write req */
	outb(KCS_CMD_IOPORT, KCS_CMD_WRITE_START);
	status = inb(KCS_STATUS_IOPORT);
	while((status & KCS_IBF)) {
		printf("Waiting for IBF to clear\n");
		sleep(1);
		status = inb(KCS_STATUS_IOPORT);
	}
	/* must be in write state */
	status = inb(KCS_STATUS_IOPORT);
	if((status & KCS_STATBITS) != KCS_WRITE ) {
		printf("Not in write state?!? 0x%x\n", status);
		exit(1);
	}
	/* clear OBF */
	status = inb(KCS_STATUS_IOPORT);
	if(status & KCS_OBF) {
		inb(KCS_DATAOUT_IOPORT);
		printf("Clearing OBF before data loop\n");
	}
	
	/* inject bytes */
	for(t=0; t<1; t++) {
		outb(KCS_DATAIN_IOPORT, a[t]);
		status = inb(KCS_STATUS_IOPORT);
		while((status & KCS_IBF)) {
			printf("Waiting for IBF to clear (data write)\n");
			sleep(1);
			status = inb(KCS_STATUS_IOPORT);
		}
		if((status & KCS_STATBITS) != KCS_WRITE) {
			printf("Not in write state (data write)\n");
			exit(1);
		}
		if(status & KCS_OBF) {
			inb(KCS_DATAOUT_IOPORT);
			printf("Clearing OBF in data loop\n");
		}
	}
	/* all done */
	outb(KCS_CMD_IOPORT, KCS_CMD_WRITE_END);
	status = inb(KCS_STATUS_IOPORT);
	if(status & KCS_OBF) {
		printf("OBF up after WRITE_END\n");
	}
	while((status & KCS_IBF)) {
		printf("Waiting for IBF to clear (end write)\n");
		sleep(1);
		status = inb(KCS_STATUS_IOPORT);
	}
	if((status & KCS_STATBITS) != KCS_WRITE) {
		printf("Not in write state (end write)\n");
		exit(1);
	}
	status = inb(KCS_STATUS_IOPORT);
	if(status & KCS_OBF) {
		inb(KCS_DATAOUT_IOPORT);
		printf("Clearing OBF for the final time\n");
	}
	/* Kick out final byte. */
	outb(KCS_DATAIN_IOPORT, a[1]);

	/* Now to read the response. */
	status = inb(KCS_STATUS_IOPORT);
	while((status & KCS_IBF)) {
		printf("Waiting for IBF to clear (end write)\n");
		sleep(1);
		status = inb(KCS_STATUS_IOPORT);
	}
	status = inb(KCS_STATUS_IOPORT);

	/* Allocate some space to handle these bytes */
	v = 0;
	bzero(out, 24);

	/* loop here until read -> idle */
	while((status & KCS_STATBITS) == KCS_READ) {
		/* suck in bytes */
		while(!(status & KCS_OBF)) {
			/* printf("Waiting for OBF (begin read)\n");*/
			sleep(1);
			status = inb(KCS_STATUS_IOPORT);
		}
		t = inb(KCS_DATAOUT_IOPORT);
		/* printf("Read byte 0x%x\n", t); */
		out[v] = t;
		v++;
		/* write next-byte, yes to DATA_IN */
		outb(KCS_DATAIN_IOPORT, KCS_CMD_READ);
		status = inb(KCS_STATUS_IOPORT);
		while((status & KCS_IBF)) {
			/* printf("Waiting for IBF to clear (read loop)\n");*/
			sleep(0.5);
			status = inb(KCS_STATUS_IOPORT);
		}
		status = inb(KCS_STATUS_IOPORT);
	}

	/* not in read anymore .. are we idle? */
	if((status & KCS_STATBITS) != KCS_IDLE) {
		printf("Escaped from READ to something else, state=0x%x\n", status);
		exit(1);	
	}
	while(!(status & KCS_OBF)) {
		printf("Waiting for OBF (going idle)\n");
		sleep(1);
		status = inb(KCS_STATUS_IOPORT);
	}
	/* Clear OBF to allow any pending interrupts to go off */
	inb(KCS_DATAOUT_IOPORT);

	/* Interpret results (or try) */
	
	printf("LUN/NetFn      : 0x%x/0x%x\n", out[0] & 0x3, out[0] >> 2);
	printf("Command        : 0x%x\n", out[1]);
	printf("Completion Code: 0x%x\n", out[2]);
	printf("Device ID      : 0x%x\n", out[3]);
	printf("Device Revision: 0x%x\n", out[4]);
	printf("Firmware Rev 1 : 0x%x\n", out[5]);
	printf("Firmware Rev 2 : 0x%x\n", out[6]);
	printf("IPMI Version   : 0x%x\n", out[7]);
	printf("Adt'l Dev Supt : 0x%x\n", out[8]);
	if(out[8] & 0x1) 
	printf("               : Sensor Device\n");
	if(out[8] & 0x2) 
	printf("               : Sensor Data Record (SDR) Repository\n");
	if(out[8] & 0x4) 
	printf("               : System Event Log (SEL)\n");
	if(out[8] & 0x8) 
	printf("               : Field-Replaceable Unit (FRU) Inventory\n");
	if(out[8] & 0x10) 
	printf("               : IPMB Event Receiver\n");
	if(out[8] & 0x20) 
	printf("               : IPMB Event Generator\n");
	if(out[8] & 0x40) 
	printf("               : IPMI Message Bridging\n");
	if(out[8] & 0x80) 
	printf("               : Chassis Device\n");
	printf("Mfr ID         : 0x%x%x%x\n", out[11], out[10], out[9]);
	printf("Product ID     : 0x%x%x\n", out[13], out[12]);
	printf("Aux FW Info    : 0x%x%x%x\n", out[16], out[15], out[14]);

	for(u=0; u<24; u++) {
		if(u%8 == 0) printf("\n");
		printf("0x%x ", out[u]);
	}
	printf("\n");

	/* All DONE!!! */
		
}

