/*-
 * Copyright (c) 2002 Doug White
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * 
 */

#include <sys/types.h>
#include <machine/cpufunc.h>
#include <machine/sysarch.h>
#include <stdio.h>
#include "kcs.h"

int
main(int argc, char** argv)
{
	int t, u, v;
	int status = 0;

	t = i386_set_ioperm(KCS_BASE_IOPORT, 0x2, 0x1);
	if(t != 0) {
		perror("i386_set_ioperm");
		exit(1);
	}

	/*
	* now we have our port, try poking the interface
	* the register looks like so: (note: MSB)
	* 7 	6	5	4	3	2	1	0
	* read-only:
	* S1	S0	OEM2	OEM1	CD#	SMS_ATN	IBF	OBF
	* data_out ---->
	* write-only:
	* command --->
	* data_in --->
	*/

	/* try to ascertain current state */
   while(1) {
	status = 0;
	status = inb(KCS_STATUS_IOPORT);
	printf("Current KCS state: 0x%x (0x%x) : ", status, (status & KCS_STATBITS));
	if((status & KCS_STATBITS) == KCS_IDLE) {
		printf("IDLE_STATE ");
	} else if((status & KCS_STATBITS) == KCS_READ) {
		printf("READ_STATE ");
	} else if((status & KCS_STATBITS) == KCS_WRITE) {
		printf("WRITE_STATE ");
	} else if((status & KCS_STATBITS) == KCS_ERROR) {
		printf("ERROR_STATE ");
	} else {
		printf("Huh? ");
	}
	if(status & KCS_IBF) {
		printf("IBF ");
	}
	if(status & KCS_OBF) {
		printf("OBF ");
	}
	if(status & KCS_OEM1) {
		printf("OEM1 ");
	}
	if(status & KCS_OEM2) {
		printf("OEM2 ");
	}
	printf("\n");
	/* printf("Current KCS statebits: 0x%x\n", status & KCS_STATBITS); */

	if( 0 && (status & KCS_OBF) & (status & KCS_SMS_ATN)) {
		/* something to read! */
		t = inb(KCS_DATAOUT_IOPORT);
		printf("Read byte 0x%x\n", t);
		/* thank you sir, may I have another? */
		status = inb(KCS_STATUS_IOPORT);
			/* READ isn't a command, it's data. :-/ */
		if( ! ( status & KCS_IBF) && (status & KCS_STATBITS) == KCS_READ) {
			printf("IBF is clear, request next byte\n");
			outb(KCS_DATAIN_IOPORT, KCS_CMD_READ);
		}
	} else if( status & KCS_IBF) {
		/* Ready for command or data! */
		if((status & KCS_STATBITS) == KCS_READ) {
			/* Still chewing on our read req */
		} 
	}
	sleep(1);	
   }	
	
}

