/*-
 * Copyright (c) 2002 Doug White
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * 
 */

/*
 *  set-ser-preboot.c
 *
 */
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include "kcs.h"
#include "kcs-common.h"
#include "ipmi-common.h"

#define KCSGET_REQUEST_LEN	5
#define KCSGET_RESPONSE_LEN	0xff

void
usage(void)
{
	printf("Usage: -c command -n NetFn [-l LUN] [-v] data...\n");
	exit(1);
}

int
main(int argc, char** argv)
{
	int t=0;
	int u=0;
	int cmdlen=0;
	int ret=0;
	int verbose=0;
	int netfn=-1;
	int lun=0;
	int command=-1;
	u_int8_t cmd[KCSGET_REQUEST_LEN];
	u_int8_t out[KCSGET_RESPONSE_LEN];
	bzero(cmd, KCSGET_REQUEST_LEN+1);
	bzero(out, KCSGET_RESPONSE_LEN+1);
	
	while((u = getopt(argc, argv, "c:l:n:v")) != -1)
	{
		switch(u) {
		case 'c':
			command=strtol(optarg, NULL, 0);
			break;
		case 'l':
			lun=strtol(optarg, NULL, 0);
			break;
		case 'n':
			netfn=strtol(optarg, NULL, 0);
			break;
		case 'v':
			verbose=1;
			break;
		case '?':
		default:
			usage();
		}
	}
	argc -= optind;
	argv += optind;
	
	/* Command and NetFn must be filled in */
	
	if(command == -1 || netfn == -1) {
		usage();
	}
        if(verbose) {
                printf("Setting NetFn %#x, Command %#x\n", netfn, command);
        }
	
	/* Get permission to use I/O portrange */
	t = i386_set_ioperm(KCS_BASE_IOPORT, 0x2, 0x1);
	if(t != 0) {
		perror("i386_set_ioperm");
		exit(1);
	}
	
	/* KCS Command Bytes */
	cmd[0] = ipmi_compose_netfn_lun(netfn, lun);
	cmd[1] = command;
	
	/* XXX I bet this is evil. */
	cmdlen=2;
	for(u=0; u < argc; u++) {
                if(verbose) {
                        printf("Adding data byte #%d: \"%s\" %#x\n", u, argv[u], argv[u]);
                }
		cmd[cmdlen] = strtol(argv[u], NULL, 0);
		cmdlen++;
	}
	if(verbose) {
                printf("Sending %d byte command\n", cmdlen);
        }
	/* Write out the command. */
	if(kcs_write(cmd, cmdlen) == -1)
	{
		printf("Error writing cmd\n");
		exit(1);
	}
	
	/* read back the response. */
	ret = kcs_read(out, sizeof(out));
	if(ret == -1) 
	{
		printf("Read failed\n");
		exit(2);
	}
	
	/* Decode output. */
	printf("LUN            : %#0.2x\n", out[0] & 0x3);
        printf("NetFn          : %#0.2x\n", out[0] >> 2);
	printf("Command        : %#0.2x\n", out[1]);
	printf("Completion Code: %#0.2x\n", (out[2] & 0xff));

	kcs_decode_packet(ret, out);
	
	if(verbose) {
		for(u=0; u<ret; u++) {
			if(u%8 == 0) printf("\n");
			printf("0x%x ", out[u]);
		}
		
		printf("\n");
	}
}

    
     
