/*-
 * Copyright (c) 2002 Doug White
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * 
 */

/*
 *  get-ser-config.c
 *
 */
#include <stdlib.h>
#include <stdio.h>
#include "kcs.h"
#include "kcs-common.h"
#include "ipmi-common.h"

#define KCSGET_REQUEST_LEN	5
#define KCSGET_RESPONSE_LEN	0xff

int
main(int argc, char** argv)
{
#ifdef DEBUG
    int u;
#endif
    int t;
    char cmd[KCSGET_REQUEST_LEN];
    char out[KCSGET_RESPONSE_LEN];
    bzero(cmd, 6);
    bzero(out, 24);
    /* Get permission to use I/O portrange */

    t = i386_set_ioperm(KCS_BASE_IOPORT, 0x2, 0x1);
    if(t != 0) {
        perror("i386_set_ioperm");
        exit(1);
    }
    /* IPMI Get LAN Config Parameters
     * NetFn Transport (0xc), Command 0x2, LUN 0x0
     */
    /* There's a lot of these, selected by param/block selectors.
     * not easy to iterate over either. */

    /* KCS Command Bytes */
    cmd[0] = ipmi_compose_netfn_lun(0xc, 0x0);
    cmd[1] = 0x10;  /* Set Serial Config Parms */

    /* Request Data Bytes */
    cmd[2] = 0x1;  /* get param, channel number */
    cmd[3] = 3;  /* parameter selector (page) */
    cmd[4] = 0x82;  /* data to set */
    
    /* Write out the command. */
    if(kcs_write(cmd, KCSGET_REQUEST_LEN) == -1)
    {
        printf("Error writing cmd\n");
        exit(1);
    }
    
    /* read back the response. */
    if(kcs_read(out, sizeof(out)) == -1)
    {
        printf("Read failed\n");
        exit(2);
    }
    
    /* Decode output. */
    printf("LUN/NetFn      : 0x%x/0x%x\n", out[0] & 0x3, out[0] >> 2);	
    printf("Command        : 0x%x\n", out[1]);
    printf("Completion Code: 0x%x\n", (out[2] & 0xff));
    printf("Parameter Rev  : 0x%x\n", out[3]);
#if 0
    printf("Device ID      : 0x%x\n", out[3]);
    printf("Device Revision: 0x%x\n", out[4]);
    printf("Firmware Rev 1 : 0x%x\n", out[5]);
    printf("Firmware Rev 2 : 0x%x\n", out[6]);
    printf("IPMI Version   : 0x%x\n", out[7]);
    printf("Adt'l Dev Supt : 0x%x\n", out[8]);
    if(out[8] & 0x1) 
    printf("               : Sensor Device\n");
    if(out[8] & 0x2) 
    printf("               : Sensor Data Record (SDR) Repository\n");
    if(out[8] & 0x4) 
    printf("               : System Event Log (SEL)\n");
    if(out[8] & 0x8) 
    printf("               : Field-Replaceable Unit (FRU) Inventory\n");
    if(out[8] & 0x10) 
    printf("               : IPMB Event Receiver\n");
    if(out[8] & 0x20) 
    printf("               : IPMB Event Generator\n");
    if(out[8] & 0x40) 
    printf("               : IPMI Message Bridging\n");
    if(out[8] & 0x80) 
    printf("               : Chassis Device\n");
    printf("Mfr ID         : 0x%x%x%x\n", out[11], out[10], out[9]);
    printf("Product ID     : 0x%x%x\n", out[13], out[12]);
    printf("Aux FW Info    : 0x%x%x%x\n", out[16], out[15], out[14]);
#endif

#ifdef DEBUG
    for(u=0; u<24; u++) {
            if(u%8 == 0) printf("\n");
            printf("0x%x ", out[u]);
    }
#endif
    printf("\n");
}

    
     
