#!/usr/local/bin/python
#
# Copyright (c) 2002 Doug White
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: entitytree.py,v 1.3 2002/06/02 02:05:26 dwhite Exp $
#
# IPMI Entity Assocation Tree builder
# Doug White

import ipmicommon
import ipmievent
import ipmisdr

# x = None

def entsort(l, r):
    if l[0] < r[0]:
        return -1
    elif l[0] > r[0]:
        return 1
    else:
        if l[1] < r[1]:
            return -1
        elif l[1] > r[1]:
            return 1
        else:
            return 0


if __name__ == "__main__":
    # global x
    x = ipmicommon.IPMISystemInterface(interface="SMIC")
    x.establish()

    sdrs = ipmisdr.download_sdr(x)

    # Keys for the entids database is a tuple of the entity id and instance
    # number. Each value is a list of sensors and objects' SDR records
    # associated with this entity.
    entids = {}

    # loop!
    for k in sdrs.keys():
        dr = sdrs[k]
        entid = dr['entity-id']
        entinst = dr['entity-instance']
        entkey = (entid, entinst)
        if entids.has_key(entkey):
            entids[entkey].append(dr)
        else:
            entids[entkey] = [dr]

    # loop B: print tree
    y = entids.keys()
    y.sort(entsort)
    for k in y:
        print "Entity 0x%x/0x%x: " % k,
        print ipmievent.entitytypelist[k[0]]
        for l in entids[k]:
            try:
                if l['record-type'] == 0x1 or l['record-type'] == 0x2:
                    stn = l.get('sensor-type-name')
                    if not stn:
                        stn = "0x%x" % l['sensor-type']
                    etn = l.get('event-type-name')
                    if not etn:
                        etn = "0x%x" % l['event-type']
                    print "   %s/%s: 0x%x/0x%x/0x%x: \"%s\"" % (stn, etn, l['sensor-owner-id'], l['sensor-owner-lun'], l['sensor-number'], l['id-string'])
                elif l['record-type'] == 0x10:
                    print "   Generic Device: 0x%x/0x%x/0x%x: \"%s\"" % (l['device-access'], l['device-lun'], l['device-address'], l['entity-id-name'])
                elif l['record-type'] == 0x11:
                    print "   FRU: 0x%x/0x%x/0x%x: \"%s:%s\"" % (l['device-access'], l['device-lun'], l['device-address'], l['device-type-name'], l['device-modifier-name'])
                elif l['record-type'] == 0x12:
                    print "   Management Controller: 0x%x" % l['device-address']
                else:
                    print "   Unknown: %s" % l
            except:
                print "   Exception: %s" % l
            
