#!/usr/local/bin/python2.1
#
# Copyright (c) 2002 Doug White
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: ipmicmd.py,v 1.15 2002/07/19 06:03:50 dwhite Exp $

import sys
import getopt
import string
import struct
import time
import ipmichassis
import ipmicommon
import ipmievent
import ipmiexception
import ipmifru
import ipmisdr
import ipmisel
import ipmisensor
import ipmitransport

x = None
verbosity = 0

def usage():
    print "usage: %s -a ipaddr|-k|-m  -c|-f F|-l|-p S|-r N|-s [-v]" % sys.argv[0]
    print "Mode:"
    print "  -a: IP Address for IPMI LAN session"
    print "  -k: KCS System Interface"
    print "  -m: SMIC System Interface"
    print ""
    print "Command:"
    print "  -c: Scan and report status on all sensors"
    print "  -f: Display Inventory FRU number F contents"
    print "  -l: Display System Event Log"
    print "  -p: Power control, S is:"
    print "      'off' = Power off"
    print "      'on' = Power on"
    print "      'cycle' = Power cycle (off for 2s, then on)"
    print "      'reset' = Reset"
    print "      'nmi' = NMI"
    print "  -r: Get Sensor Reading for sensor device N"
    print "  -s: Dump SDR Repository"
    print ""
    print "Options:"
    print "  -v: Verbosity"

def toomanyopts():
    print "Must choose either -c, -f, -l, -p, -r, or -s option"
    usage()
    sys.exit(1)

def toomanymodes():
    print "Must choose either -a, -k, or -m option"
    usage()
    sys.exit(1)
    
def pbytes(ret):
    for y in ret:
        print "0x%x ," % ord(y)

def utos(val, bits):
    if val & (2**(bits-1)):
        return -((~val & 2**(bits-1)-1)+1)
    else:
        return val

def dopower(arg):
    # power control
    ipmichassis.control(x, arg)

        
def doseldump():

    y = ipmisel.download_sel(x)
    z = ipmisel.format_sels(x, y)
    print string.join(z, '\n')
    return
    

### Sensor Reading ####################################################
def dosensread(arg, arg2):

    sensor = int(arg, 0)
    if arg2:
        dev = int(arg2, 0)
    else:
        dev = 0x20
    sdrs = ipmisdr.download_sdr(x)

    rec = sdrs[(0x1, dev, 0x0, sensor)]

    if rec['sdr-version'] == 0x90:
        print "Version 0.9 sensors are not supported."
        return

    sensdata = ipmisensor.get_sensor_reading(x, dev, 0x0, sensor)
    out = ipmisensor.convert_raw(sensdata[0], rec)

    print "%s = %s" % (rec['id-string'], out)

    
### SDR Dump #####################################################

def dosdrdump():
    # Pull out the SDR Repo Info

    ret = x.send(0x20, 0x0, 0x81, 0x0, 1, 0xa, 0x20, '')

    retcode, sdrver, reclo, rechi, freelo, freehi, addts, erasets, opsup = struct.unpack('<BBBBBBLLB', ret)

    print "SDR Repository Information:"
    print "SDR Version          : 0x%x" % sdrver
    print "Record Count         : %d" % ((rechi << 8) + reclo)
    print "Bytes Free           : %d" % ((freehi << 8) + freelo)
    print "Last Record Added    : %s" % time.ctime(addts)
    print "Last Record Modified : %s" % time.ctime(erasets)
    print "Operational Support  : 0x%x" % opsup
    if opsup & 0x1:
        print "                     : Get SDR Allcation Info cmd supported"
    if opsup & 0x2:
        print "                     : Reserve SDR Repo cmd supported"
    if opsup & 0x4:
        print "                     : Partial Add SDR cmd supported"
    if opsup & 0x8:
        print "                     : Delete SDR cmd supported"
    if opsup & 0x20 and opsup & 0x40 == 0:
        print "                     : Non-modal SDR Update supported"
    if opsup & 0x20 == 0 and opsup & 0x40:
        print "                     : Modal SDR Update supported"
    if opsup & 0x20 and opsup & 0x40:
        print "                     : Both Modal and Non-modal update supported"
    if opsup & 0x80:
        print "                     : Overflow Flag Set"


    # Grab the repo
    sdrs = ipmisdr.download_sdr(x)

    for k in sdrs.keys():
        s = sdrs[k]
        print "------------------------------------------------------------"
        print "Record Type                      : 0x%x" % s['record-type']
        print "Record Version                   : 0x%x" % s['sdr-version']
        if s['record-type'] == 0x1:
            print "ID String                        : %s" % s['id-string']
            print "Sensor Owner ID/Lun/Number       : 0x%x/0x%x/0x%x" % (s['sensor-owner-id'], s['sensor-owner-lun'], s['sensor-number'])
            print "Entity ID/Instance               : 0x%x/0x%x" % (s['entity-id'], s['entity-instance'])
            
            print "                                 : %s" % s['entity-id-name']
            scap = s['sensor-capabilities']
            scaphyst = s['sensor-capability-hysteresis']
            scapthresh = s['sensor-capability-thresh']
            scapevent = s['sensor-capability-events']
            print "Sensor Capabilities              : 0x%x" % scap
            if scap & 0x80:
                print "                                 : Ignore if Entity not present"
            if scap & 0x40:
                print "                                 : Auto re-arming"
            if scaphyst == 0:
                print "                                 : Hysteresis unsupported"
            if scaphyst == 1:
                print "                                 : Hysteresis readable"
            if scaphyst == 2:
                print "                                 : Hysteresis read/write"
            if scaphyst == 3:
                print "                                 : Hysteresis built-in"
            if scapthresh == 0:
                print "                                 : No thresholds"
            if scapthresh == 1:
                print "                                 : Thresholds readable"
            if scapthresh == 2:
                print "                                 : Thresholds read/write"
            if scapthresh == 3:
                print "                                 : Thresholds built-in"
            if scapevent == 0:
                print "                                 : Per-threshold event control"
            if scapevent == 1:
                print "                                 : Entire sensor event control only"
            if scapevent == 2:
                print "                                 : Global disable only"
            if scapevent == 3:
                print "                                 : Sensor does not send events"
            print "Sensor Type                      : 0x%x" % s['sensor-type']
            if s.has_key('sensor-type-name'):
                print "                                 : %s" % s['sensor-type-name']
            print "Event Type                       : 0x%x" % s['event-type']
            if s.has_key('event-type-name'):
                print "                                 : %s" % s['event-type-name']
            print "Assertion Event/Thresh Read Mask : 0x%x" % s['assert-event-mask']
            print "Deassert Event/Upper Thresh Mask : 0x%x" % s['deassert-event-mask']
            print "Disc Read/Read/Write Thresh Mask : 0x%x" % s['discrete-reading-mask']
            print "Sensor Units                     : 0x%x" % s['sensor-units-1']
            adf = s['analog-data-format']
            rate = s['rate-unit']
            if adf == 0:
                print "                                 : Unsigned"
            if adf == 1:
                print "                                 : 1's Comp Signed"
            if adf == 2:
                print "                                 : 2's Comp Signed"
            if adf == 3:
                print "                                 : Non-numeric reading"
            if rate == 0:
                print "                                 : No rate unit"
            if rate == 1:
                print "                                 : Per microsend"
            if rate == 2:
                print "                                 : Per milisecond"
            if rate == 3:
                print "                                 : Per second"
            if rate == 4:
                print "                                 : Per minute"
            if rate == 5:
                print "                                 : Per hour"
            if rate == 6:
                print "                                 : Per day"
            mut = s['modifier-unit-type']
            if mut == 0:
                print "                                 : No rate unit"
            if mut == 1:
                print "                                 : Basic Divided by Modifier Unit"
            if mut == 2:
                print "                                 : Basic Multipled by Modifier Unit"
            if s['unit-percentage']:
                print "                                 : Percentage"
            print "Base Unit                        : %s" % s['base-unit']
            print "Modifier Unit                    : %s" % s['modifier-unit']
            print "Linearization                    : %d" % s['linearization']
            print "M coefficient                    : %d" % s['m']
            print "B scalar                         : %d" % s['b']
            print "Tolerance                        : %d" % s['tolerance']
            print "Accuracy                         : %d" % s['accuracy']
            print "R/B Exponent                     : %d/%d" % (s['r-exponent'], s['b-exponent'])
            achar = s['analog-characteristics']
            print "Analog characteristics           : 0x%x" % achar
            if achar & 0x4:
                print "                                 : Normal Minumum specified"
            if achar & 0x2:
                print "                                 : Normal Maximum specified"
            if achar & 0x1:
                print "                                 : Nominal Reading specified"
            print "Nominal Reading                  : %s" % ipmisensor.convert_raw(s['nominal-reading'], s)
            print "Normal Maxiumum                  : %s" % ipmisensor.convert_raw(s['normal-maximum'], s)
            print "Normal Minumum                   : %s" % ipmisensor.convert_raw(s['normal-minimum'], s)
            print "Sensor Maximum Reading           : %s" % ipmisensor.convert_raw(s['sensor-maximum-reading'], s)
            print "Sensor Minumum Reading           : %s" % ipmisensor.convert_raw(s['sensor-minimum-reading'], s)
            print "Upper Non-recoverable Threshold  : %s" % ipmisensor.convert_raw(s['upper-non-recoverable-thresh'], s)
            print "Upper Critical Threshold         : %s" % ipmisensor.convert_raw(s['upper-critical-thresh'], s)
            print "Upper Non-critical Threshold     : %s" % ipmisensor.convert_raw(s['upper-non-critical-thresh'], s)
            print "Lower Non-recoverable Threshold  : %s" % ipmisensor.convert_raw(s['lower-non-recoverable-thresh'], s)
            print "Lower Critical Threshold         : %s" % ipmisensor.convert_raw(s['lower-critical-thresh'], s)
            print "Lower Non-critical Threshold     : %s" % ipmisensor.convert_raw(s['lower-non-critical-thresh'], s)
            print "Postive-going Hysteresis         : %d" % s['positive-going-hysteresis']
            print "Negative-going Hysteresis        : %d" % s['negative-going-hysteresis']
            print "ID Type/Length (packed)          : 0x%x" % s['id-type']

##            if s.has_key('raw-b'):
##                print "B: 0x%x" % s['raw-b']
##            if s.has_key('raw-b-accuracy'):
##                print "BACC: 0x%x" % s['raw-b-accuracy']

        elif s['record-type'] == 0x2:
            print "ID String                        : %s" % s['id-string']
            print "Sensor Owner ID/Lun/Number       : 0x%x/0x%x/0x%x" % (s['sensor-owner-id'], s['sensor-owner-lun'], s['sensor-number'])
            print "Entity ID/Instance               : 0x%x/0x%x" % (s['entity-id'], s['entity-instance'])
            
            print "                                 : %s" % s['entity-id-name']
            scap = s['sensor-capabilities']
            scaphyst = s['sensor-capability-hysteresis']
            scapthresh = s['sensor-capability-thresh']
            scapevent = s['sensor-capability-events']
            print "Sensor Capabilities              : 0x%x" % scap
            if scap & 0x80:
                print "                                 : Ignore if Entity not present"
            if scap & 0x40:
                print "                                 : Auto re-arming"
            if scaphyst == 0:
                print "                                 : Hysteresis unsupported"
            if scaphyst == 1:
                print "                                 : Hysteresis readable"
            if scaphyst == 2:
                print "                                 : Hysteresis read/write"
            if scaphyst == 3:
                print "                                 : Hysteresis built-in"
            if scapthresh == 0:
                print "                                 : No thresholds"
            if scapthresh == 1:
                print "                                 : Thresholds readable"
            if scapthresh == 2:
                print "                                 : Thresholds read/write"
            if scapthresh == 3:
                print "                                 : Thresholds built-in"
            if scapevent == 0:
                print "                                 : Per-threshold event control"
            if scapevent == 1:
                print "                                 : Entire sensor event control only"
            if scapevent == 2:
                print "                                 : Global disable only"
            if scapevent == 3:
                print "                                 : Sensor does not send events"
            print "Sensor Type                      : 0x%x" % s['sensor-type']
            if s.has_key('sensor-type-name'):
                print "                                 : %s" % s['sensor-type-name']
            print "Event Type                       : 0x%x" % s['event-type']
            if s.has_key('event-type-name'):
                print "                                 : %s" % s['event-type-name']
            print "Assertion Event/Thresh Read Mask : 0x%x" % s['assert-event-mask']
            print "Deassert Event/Upper Thresh Mask : 0x%x" % s['deassert-event-mask']
            print "Disc Read/Read/Write Thresh Mask : 0x%x" % s['discrete-reading-mask']
            print "Sensor Units                     : 0x%x" % s['sensor-units-1']
            adf = s['analog-data-format']
            rate = s['rate-unit']
            if adf == 0:
                print "                                 : Unsigned"
            if adf == 1:
                print "                                 : 1's Comp Signed"
            if adf == 2:
                print "                                 : 2's Comp Signed"
            if adf == 3:
                print "                                 : Non-numeric reading"
            if rate == 0:
                print "                                 : No rate unit"
            if rate == 1:
                print "                                 : Per microsend"
            if rate == 2:
                print "                                 : Per milisecond"
            if rate == 3:
                print "                                 : Per second"
            if rate == 4:
                print "                                 : Per minute"
            if rate == 5:
                print "                                 : Per hour"
            if rate == 6:
                print "                                 : Per day"
            mut = s['modifier-unit-type']
            if mut == 0:
                print "                                 : No rate unit"
            if mut == 1:
                print "                                 : Basic Divided by Modifier Unit"
            if mut == 2:
                print "                                 : Basic Multipled by Modifier Unit"
            if s['unit-percentage']:
                print "                                 : Percentage"
            print "Base Unit                        : %s" % s['base-unit']
            print "Modifier Unit                    : %s" % s['modifier-unit']
            print "Sensor Record Sharing            : 0x%x" % s['sensor-record-sharing']
            print "ID String Modifier Type          : 0x%x" % s['id-string-modifier-type']
            if s['id-string-modifier-type']:
                print "                                 : Alpha"
            else:
                print "                                 : Numeric"
            print "Share Count                      : 0x%x" % s['share-count']
            print "Entity Instance Sharing          : 0x%x" % s['entity-instance-sharing']
            if s['entity-instance-sharing']:
                print "                                 : Increment per record"
            else:
                print "                                 : Same for all records"
            print "ID String Modifier Offset        : 0x%x" % s['id-string-modifier-offset']
            print "Positive-going Hysteresis        : %d" % s['positive-going-hysteresis']
            print "Negative-going Hysteresis        : %d" % s['negative-going-hysteresis']

        elif s['record-type'] == 0x10:
            if s['sdr-version'] == 0x90:
                print "ID String                        : %s" % s['id-string']
                print "Device Access Address            : 0x%x" % s['device-access']
                print "Device Slave Address             : 0x%x" % s['device-address']
                print "Device Mode                      : %s" % s['device-type']
                print "Private Bus ID                   : 0x%x" % s['private-bus-id']
                if s.has_key('device-type-name') and s.has_key('device-modifier-name'):
                    print "Device Type                      : %s: %s" % (s['device-type-name'], s['device-modifier-name'])
                else:
                    print "Device Type                      : 0x%x/0x%x" % (s['device-type-code'], s['device-type-modifier'])
                if s.has_key('entity-id-name'):
                    print "Entity ID                        : %s" % s['entity-id-name']
                else:
                    print "Entity ID                        : 0x%x" % s['entity-id']
                print "Entity Instance                  : 0x%x" % s['entity-instance']
                print "Associated Entity ID             : 0x%x" % s['associated-entity-id']
                print "Firmware Revision                : 0x%x" % s['firmware-revision']
                
            else:
                print "ID String                        : %s" % s['id-string']
                print "Device Access Address            : 0x%x" % s['device-access']
                print "Device Slave Address             : 0x%x" % s['device-address']
                print "Channel Number                   : 0x%x" % s['channel-number']
                print "Device LUN                       : 0x%x" % s['device-lun']
                print "Device Mode                      : %s" % s['device-type']
                print "Private Bus ID                   : 0x%x" % s['private-bus-id']
                if s.has_key('device-type-name'):
                    print "Device Type                      : %s: %s" % (s['device-type-name'], s['device-modifier-name'])
                else:
                    print "Device Type                      : 0x%x/0x%x" % (s['device-type-code'], s['device-type-modifier'])
                if s.has_key('entity-id-name'):
                    print "Entity ID                        : %s" % s['entity-id-name']
                else:
                    print "Entity ID                        : 0x%x" % s['entity-id']
                print "Entity Instance                  : 0x%x" % s['entity-instance']
            
        elif s['record-type'] == 0x11:
            print "ID String                        : %s" % s['id-string']
            print "Device Access Address            : 0x%x" % s['device-access']
            print "Device Slave Address             : 0x%x" % s['device-address']
            print "Device LUN                       : 0x%x" % s['device-lun']
            print "Access Word                      : 0x%x" % s['access-info']
            print "Device Mode                      : %s" % s['device-type']
            print "Private Bus ID                   : 0x%x" % s['private-bus-id']
            print "Channel Number                   : 0x%x" % s['channel-number']
            if s.has_key('device-type-name'):
                print "Device Type                      : %s: %s" % (s['device-type-name'], s['device-modifier-name'])
            else:
                print "Device Type                      : 0x%x/0x%x" % (s['device-type-code'], s['device-type-modifier'])
            if s.has_key('entity-id-name'):
                print "Entity ID                        : %s" % s['entity-id-name']
            else:
                print "Entity ID                        : 0x%x" % s['entity-id']
            print "Entity Instance                  : 0x%x" % s['entity-instance']
            
        elif s['record-type'] == 0x12:
            print "ID String                        : %s" % s['id-string']
            print "Device Address                   : 0x%x" % s['device-address']
            print "Channel Number                   : 0x%x" % s['channel-number']
            print "Device Capabilities              : 0x%x" % s['device-capabilities-word']
            for i in s['device-capabilities']:
                print "                                 : %s" % i
            print "Event Generation                 : %s" % s['event-generation']
            if s.has_key('entity-id-name'):
                print "Entity ID                        : %s" % s['entity-id-name']
            else:
                print "Entity ID                        : 0x%x" % s['entity-id']
            print "Entity Instance                  : 0x%x" % s['entity-instance']

### Sensor Scan #####################################################

def sortsensscan(l, r):
    if l[0] < r[0]:
        return -1
    elif l[0] > r[0]:
        return 1
    else:
        return 0

def dosensscan():
    
    sdrs = ipmisdr.download_sdr(x)

    # We want to loop over the sensor SDRs, grab their values,
    # compare to the SDR limits, and print.

    out = []

    for sk in sdrs.keys():
        if sdrs[sk]['sdr-version'] == 0x90:
            continue
        if sk[0] != 0x1 and sk[0] != 0x2:
            continue
        if sdrs[sk].get('event-type-name') != 'Threshold':
            continue

        try:
            value, crit = ipmisensor.get_and_compare_sensor(x, sk[1], sk[2], sk[3], sdrs[sk])
        except ipmiexception.IPMISensorDisabled:
            print "sensor 0x%x disabled" % sdrs[sk]['sensor-number']
            out.append((sdrs[sk]['sensor-number'], sdrs[sk]['id-string'], 'Disabled', ''))
            continue
        out.append((sdrs[sk]['sensor-number'], sdrs[sk]['id-string'], crit[1], value))

    out.sort(sortsensscan)

    for y in out:
        print "%2s: %-20s : %-12s : %s" % y


### FRU Read #####################################################

def dofruread(arg):

    fru = int(arg, 0)

    # temp: arg == from device address, assume device-access == 0x10 and lun == 0
    sdrs = ipmisdr.download_sdr(x)
    fsdr = sdrs[(0x11, 0x10, fru)]

    rec = ipmifru.get_fru_data(x, fsdr)
    print "Product Manufacturer : %s" % rec['product-manufacturer']
    print "Product Name         : %s" % rec['product-name']
    print "Product Part Number  : %s" % rec['product-part-number']
    print "Product Serial Number: %s" % rec['product-serial-number']
    print "Asset Tag            : %s" % rec['product-asset-tag']
    print "Chassis Type         : %s" % rec['chassis-type-name']
    print "Chassis Part Number  : %s" % rec['chassis-part-number']
    print "Chassis Serial Number: %s" % rec['chassis-serial-number']
    print "Board Name           : %s" % rec['board-product-name']
    print "Board Manufacturer   : %s" % rec['board-manufacturer']
    print "Manufacture Date     : %s" % time.ctime(rec['board-manufactured'])
    print "Board Part Number    : %s" % rec['board-part-number']
    print "Board Serial Number  : %s" % rec['board-serial-number']

## MAIN #################################################

def doit():

    global x
    global verbosity
    function = None
    arg = None
    arg2 = None
    # x = None

    try:
        optlist, args = getopt.getopt(sys.argv[1:], 'a:cf:i:kmlp:r:sv')
    except getopt.GetoptError:
        print "Bad option"
        usage()
        sys.exit(1)
        
    for o, a in optlist:
        
        if o == "-a":
            if x == None:
                x = ipmicommon.IPMISession(a)
            else:
                toomanymodes()
        if o == "-k":
            if x == None:
                x = ipmicommon.IPMISystemInterface(interface="KCS")
            else:
                toomanymodes()
        if o == "-m":
            if x == None:
                x = ipmicommon.IPMISystemInterface(interface="SMIC")
            else:
                toomanymodes()
        if o == "-c":
            if function == None:
                function = o
            else:
                toomanyopts()
        if o == "-f":
            if function == None:
                function = o
                arg = a
            else:
                toomanyopts()
        if o == "-i":
                arg2 = a
        if o == "-l":
            if function == None:
                function = o
            else:
                toomanyopts()
        if o == "-p":
            if function == None:
                function = o
                arg = a
            else:
                toomanyopts()
        if o == "-r":
            if function == None:
                function = o
                arg = a
        if o == "-s":
            if function == None:
                function = o
            else:
                toomanyopts()
        if o == "-v":
            verbosity = verbosity + 1

    if function == None:
        usage()
        sys.exit(1)

    if x == None:
        usage()
        sys.exit(1)

    x.establish()

    if function == "-p":
        dopower(arg)
    elif function == "-l":
        doseldump()
    elif function == "-r":
        dosensread(arg, arg2)
    elif function == "-s":
        dosdrdump()
    elif function == "-c":
        dosensscan()
    elif function == "-f":
        dofruread(arg)
    else:
        print "eh?"
        usage()
        sys.exit(1)

   

if __name__ == "__main__":
    doit()
    
