#
# Copyright (c) 2002 Doug White
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: ipmidevice.py,v 1.4 2002/07/20 02:31:55 dwhite Exp $
#
# IPMI Device decoding
#

# Decode a device type and modifier.

def decode_device(dt, dm):

    devtype = ""
    devmod = ""

    if dt == 0x0:
        devtype = "v0.9 IPMB Device"
    elif dt == 0x1:
        devtype = "v0.9 SMM Card IPMB Device"
    elif dt == 0x2:
        devtype = "DS1624 temperature sensor"
    elif dt == 0x3:
        devtype = "DS1621 tempearture sensor"
    elif dt == 0x4:
        devtype = "LM75 temperature sensor"
    elif dt == 0x5:
        devtype = "Heceta ASIC"
        if dm == 0x0:
            devmod = "Heceta 1 LM78"
        elif dm == 0x1:
            devmod = "Heceta 2 LM79"
        elif dm == 0x2:
            devmod = "LM80"
        elif dm == 0x3:
            devmod = "Heceta 3 LM81, ADM9240, DS1780"
        elif dm == 0x4:
            devmod = "Heceta 4"
        elif dm == 0x5:
            devmod = "Heceta 5"
    elif dt == 0x8:
        devtype = "24C01 EEPROM"
        if dm == 0x1:
            devmod = "DIMM Memory ID"
        if dm == 0x2:
            devmod = "IPMI FRU Inventory"
        if dm == 0x3:
            devmod = "System Processor Cartridge FRU/PIROM"
    elif dt == 0x9:
        devtype = "24C02 EEPROM"
        if dm == 0x1:
            devmod = "DIMM Memory ID"
        if dm == 0x2:
            devmod = "IPMI FRU Inventory"
        if dm == 0x3:
            devmod = "System Processor Cartridge FRU/PIROM"
    elif dt == 0xa:
        devtype = "24C04 EEPROM"
        if dm == 0x1:
            devmod = "DIMM Memory ID"
        if dm == 0x2:
            devmod = "IPMI FRU Inventory"
        if dm == 0x3:
            devmod = "System Processor Cartridge FRU/PIROM"
    elif dt == 0xb:
        devtype = "24C08 EEPROM"
        if dm == 0x1:
            devmod = "DIMM Memory ID"
        if dm == 0x2:
            devmod = "IPMI FRU Inventory"
        if dm == 0x3:
            devmod = "System Processor Cartridge FRU/PIROM"
    elif dt == 0xc:
        devtype = "24C16 EEPROM"
        if dm == 0x1:
            devmod = "DIMM Memory ID"
        if dm == 0x2:
            devmod = "IPMI FRU Inventory"
        if dm == 0x3:
            devmod = "System Processor Cartridge FRU/PIROM"
    elif dt == 0xd:
        devtype = "24C17 EEPROM"
        if dm == 0x1:
            devmod = "DIMM Memory ID"
        if dm == 0x2:
            devmod = "IPMI FRU Inventory"
        if dm == 0x3:
            devmod = "System Processor Cartridge FRU/PIROM"
    elif dt == 0xe:
        devtype = "24C32 EEPROM"
        if dm == 0x1:
            devmod = "DIMM Memory ID"
        if dm == 0x2:
            devmod = "IPMI FRU Inventory"
        if dm == 0x3:
            devmod = "System Processor Cartridge FRU/PIROM"
    elif dt == 0xf:
        devtype = "24C64 EEPROM"
        if dm == 0x1:
            devmod = "DIMM Memory ID"
        if dm == 0x2:
            devmod = "IPMI FRU Inventory"
        if dm == 0x3:
            devmod = "System Processor Cartridge FRU/PIROM"
    elif dt == 0x10:
        devtype = "FRU Inventory Device-controlled"
        if dm == 0x0:
            devmod = "IPMI FRU Inventory"
        if dm == 0x1:
            devmod = "DIMM Memory ID"
        if dm == 0x2:
            devmod = "IPMI FRU Inventory"
        if dm == 0x3:
            devmod = "System Processor Cartridge FRU/PIROM"
    elif dt == 0x14:
        devtype = "PCF 8570 256 byte RAM"
    elif dt == 0x15:
        devtype = "PCF 8573 clock/calendar"
    elif dt == 0x16:
        devtype = "PCF 8574A I/O port"
    elif dt == 0x17:
        devtype = "PCF 8583 clock/calendar"
    elif dt == 0x18:
        devtype = "PCF 8593 clock/calendar"
    elif dt == 0x19:
        devtype = "Clock/calendar"
    elif dt == 0x1a:
        devtype = "PCF 8591 D/A, A/D converter"
    elif dt == 0x1b:
        devtype = "I/O port"
    elif dt == 0x1c:
        devtype = "A/D converter"
    elif dt == 0x1d:
        devtype = "D/A converter"
    elif dt == 0x1e:
        devtype = "A/D, D/A converter"
    elif dt == 0x1f:
        devtype = "LCD controller"
    elif dt == 0x20:
        devtype = "Core logic device"
    elif dt == 0x21:
        devtype = "LMC 6874 Intelligent Battery controller"
    elif dt == 0x22:
        devtype = "Intelligent Battery controller"
    elif dt == 0x23:
        devtype = "Combo Management ASIC"
    elif dt == 0x24:
        devtype = "Maxim 1617 temperature sensor"
    elif dt == 0xbf:
        devtype = "Other unspecified device"
    elif dt <= 0xc0 and dt >= 0xff:
        devtype = "OEM"

    return (devtype, devmod)
