#
# Copyright (c) 2002 Doug White
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: ipmievent.py,v 1.8 2002/07/20 02:31:55 dwhite Exp $
#
# IPMI Event decoding
#


# Take an event's data and return a pair of strings describing it

sensortypelist = ["reserved", "Temperature", "Voltage", "Current", "Fan", "Physical Security", "Platform Security",  "Processor", "Power Supply", "Power Unit", "Cooling Device","Other Unit-based Sensor","Memory","Drive Bay", "POST Memory","Boot Progress","SEL Disabled","Watchdog 1", "System Event","Critical Interrupt","Button","Module/Board","Microcontroller","Add-in Card","Chassis","Chipset","Other FRU","Interconnect","Terminator","System Boot","Boot Error","OS Boot","OS Critical Stop","Slot/Connector","ACPI Power State","Watchdog 2","Platform Alert","Entity Presence","Monitor ASIC","LAN","Management Health", "Battery"]

eventtypelist = ["Unspecified","Threshold","Usage State","State","Predictive Failure", "Limit","Performance","Severity","Device Presence","Device Enable","Device State","Redundancy State","ACPI Power State","Sensor-specific"]

entitytypelist = ['Unspecified', 'Other', 'unknown', 'processor', 'disk or disk bay', 'peripheral bay', 'system management module', 'system board', 'memory module', 'processor module', 'power supply', 'add-in card', 'front panel board', 'back panel board', 'power system board', 'drive backplane', 'system internal expansion board', 'other system board', 'processor board', 'power unit', 'power module', 'power management/distribution board', 'chassis back panel board', 'system chassis', 'sub-chassis', 'other chassis board', 'disk drive bay', 'peripheral bay', 'device bay', 'fan/cooling device', 'cooling unit', 'interconnect', 'memory device', 'system management software', 'BIOS', 'operating system', 'system bus', 'group', 'remote management device', 'external environment', 'battery']

unittypelist = ["unspecified", "Celsius", "Farenheit", 'Kelvin', 'Volt', 'Amp', 'Watt', 'Joule', 'Coulomb', 'VA', 'Nits', 'Lumen', 'Lux', 'Candela', 'kPa', 'psi', 'Newton', 'CFM', 'RPM', 'Hz', 'microsecond', 'millisecond', 'second', 'minute', 'hour', 'day', 'week', 'mil', 'inches', 'feed', 'cubic inches', 'cubic feet', 'millimeter', 'centimeter', 'meter', 'cubic centimeter', 'cubic meter', 'litre', 'fluid ounce', 'radian', 'steradian', 'revolution', 'cycle', 'gravity', 'ounce', 'pound', 'foot-pound', 'ounce-inch', 'gauss', 'gilbert', 'henry', 'millihenry', 'farad', 'microfarad', 'ohm', 'siemen', 'mole', 'becquerel', 'parts per million', 'reserved', 'decibels', 'DbA', 'DbC', 'gray', 'sievert', 'color temperature in Kelvin', 'bit', 'kilobit', 'megabit', 'gigabit', 'byte', 'kilobyte', 'megabyte', 'gigabyte', 'word', 'dword', 'qword', 'line', 'hit', 'miss', 'retry', 'reset', 'overflow', 'underrun', 'collision', 'packet' 'message', 'character', 'error', 'correctable error', 'uncorrectable error']

def decode_event(etype, eoff, stype, data1, data2, data3):

    #print "Decoding event: etype=0x%x eoff=0x%x stype=0x%x" % (etype, eoff, stype)

    etypestr = ""
    especstr = ""
    
    if etype == 0x0:
        etypestr = "Unspecified"
    if etype == 0x1:
        etypestr = "Threshold"
        if eoff == 0x0:
            especstr = "Lower Non-critical going low"
        elif eoff == 0x1:
            especstr = "Lower Non-critical going high"
        elif eoff == 0x2:
            especstr = "Lower Critical going low"
        elif eoff == 0x3:
            especstr = "Lower Critical going high"
        elif eoff == 0x4:
            especstr = "Lower Non-recoverable going low"
        elif eoff == 0x5:
            especstr = "Lower Non-recoverable going high"
        elif eoff == 0x6:
            especstr = "Upper Non-critical going low"
        elif eoff == 0x7:
            especstr = "Upper Non-critical going high"
        elif eoff == 0x8:
            especstr = "Upper Critical going low"
        elif eoff == 0x9:
            especstr = "Upper Critical going high"
        elif eoff == 0xa:
            especstr = "Upper Non-recoverable going low"
        elif eoff == 0xb:
            especstr = "Upper Non-recoverable going high"
    if etype == 0x2:
        etypestr = "Usage State"
        if eoff == 0x0:
            especstr = "Transition to idle"
        elif eoff == 0x1:
            especstr = "Transition to active"
        elif eoff == 0x2:
            especstr = "Transition to busy"
    if etype == 0x3:
        etypestr = "State"
        if eoff == 0x0:
            especstr = "Deasserted"
        elif eoff == 0x1:
            especstr = "Asserted"
    if etype == 0x4:
        etypestr = "Predictive Failure"
        if eoff == 0x0:
            especstr = "Deasserted"
        elif eoff == 0x1:
            especstr = "Asserted"
    if etype == 0x5:
        etypestr = "Limit"
        if eoff == 0x0:
            especstr = "Not exceeded"
        elif eoff == 0x1:
            especstr = "Exceeded"
    if etype == 0x6:
        etypestr = "Performance"
        if eoff == 0x0:
            especstr = "Met"
        elif eoff == 0x1:
            especstr = "Lags"
    if etype == 0x7:
        etypestr = "Severity"
        if eoff == 0x0:
            especstr = "Transition to OK"
        elif eoff == 0x1:
            especstr = "Transition to Non-critical from OK"
        elif eoff == 0x2:
            especstr = "Transition to Critical from less severe"
        elif eoff == 0x3:
            especstr = "Transition to Non-recoverable from less severe"
        elif eoff == 0x4:
            especstr = "Transition to Non-critical from more severe"
        elif eoff == 0x5:
            especstr = "Transition to Critical from Non-recoverable"
        elif eoff == 0x6:
            especstr = "Transition to Non-recoverable"
        elif eoff == 0x7:
            especstr = "Monitor"
        elif eoff == 0x8:
            especstr = "Informational"
    if etype == 0x8:
        etypestr = "Device Presence"
        if eoff == 0x0:
            especstr = "Absent"
        if eoff == 0x1:
            especstr = "Present"
        if eoff == 0x2:
            especstr = "Something Else"
    if etype == 0x9:
        etypestr = "Device Enable"
        if eoff == 0x0:
            especstr = "Disabled"
        if eoff == 0x1:
            especstr = "Enabled"
    if etype == 0xa:
        etypestr = "Device State"
        if eoff == 0x0:
            especstr = "Transition to Running"
        elif eoff == 0x1:
            especstr = "Transition to In Test"
        elif eoff == 0x2:
            especstr = "Transition to Power Off"
        elif eoff == 0x3:
            especstr = "Transition to On Line"
        elif eoff == 0x4:
            especstr = "Transition to Off Line"
        elif eoff == 0x5:
            especstr = "Transition to Off Duty"
        elif eoff == 0x6:
            especstr = "Transition to Degraded"
        elif eoff == 0x7:
            especstr = "Transition to Power Save"
        elif eoff == 0x8:
            especstr = "Install Error"
    if etype == 0xb:
        etypestr = "Redundancy State"
        # the strings for these are unclear in the spec without needing
        # further explanation.
        if eoff == 0x0:
            especstr = "Fully Redundant"
        elif eoff == 0x1:
            especstr = "Redundancy Lost"
        elif eoff == 0x2:
            especstr = "Redundancy Degraded"
        elif eoff == 0x3:
            especstr = "Non-redundant"
        elif eoff == 0x4:
            especstr = "Non-redundant"
        elif eoff == 0x5:
            especstr = "Insufficient resources"
        elif eoff == 0x6:
            especstr = "Redundancy Degraded"
        elif eoff == 0x7:
            especstr = "Redundancy Degraded"
    if etype == 0xc:
        etypestr = "ACPI Power State"
        if eoff == 0x0:
            especstr = "D0"
        elif eoff == 0x1:
            especstr = "D1"
        elif eoff == 0x2:
            especstr = "D2"
        elif eoff == 0x3:
            especstr = "D3"
    if etype == 0x6f:
        etypestr = "Sensor-specific"
        if stype == 0x1:
            etypestr = "Temperature"
        if stype == 0x2:
            etypestr = "Voltage"
        if stype == 0x3:
            etypestr = "Current"
        if stype == 0x4:
            etypestr = "Fan"
        if stype == 0x5:
            etypestr = "Physical Security"
            if eoff == 0x0:
                especstr = "General Chassis Intrusion"
            if eoff == 0x1:
                especstr = "Drive Bay Intrusion"
            if eoff == 0x2:
                especstr = "I/O Card Area Intrusion"
            if eoff == 0x3:
                especstr = "Processor Area Intrusion"
            if eoff == 0x4:
                especstr = "LAN Unplugged"
            if eoff == 0x5:
                especstr = "Unauthorized Dock/Undock"
            if eoff == 0x6:
                especstr = "Fan Area Intrusion"
        if stype == 0x6:
            etypestr = "Platform Security"
            if eoff == 0x0:
                especstr = "Front Panel Lockout Violation"
            if eoff == 0x1:
                especstr = "User Boot Password Violation"
            if eoff == 0x2:
                especstr = "Setup Boot Password Violation"
            if eoff == 0x3:
                especstr = "Network Boot Password Violation"
            if eoff == 0x4:
                especstr = "Boot Password Violation"
            if eoff == 0x5:
                especstr = "Out-of-band Access Password Violation"
        if stype == 0x7:
            etypestr = "Processor"
            if eoff == 0x0:
                especstr = "IERR"
            if eoff == 0x1:
                especstr = "Thermal Trip"
            if eoff == 0x2:
                especstr = "FRB1/BIST Failure"
            if eoff == 0x3:
                especstr = "FRB2/Hang in POST"
            if eoff == 0x4:
                especstr = "FRB3/CPU Startup Failure"
            if eoff == 0x5:
                especstr = "Configuration Error"
            if eoff == 0x6:
                especstr = "Uncorrectable CPU-complex Error"
            if eoff == 0x7:
                especstr = "Present"
            if eoff == 0x8:
                especstr = "Disabled"
            if eoff == 0x9:
                especstr = "Terminated"
        if stype == 0x8:
            etypestr = "Power Supply"
            if eoff == 0x0:
                especstr = "Presence detected"
            elif eoff == 0x1:
                especstr = "Power supply failure"
            elif eoff == 0x2:
                especstr = "Predictive failure"
            elif eoff == 0x3:
                especstr = "AC lost"
            elif eoff == 0x4:
                especstr = "AC lost or out of range"
            elif eoff == 0x5:
                especstr = "AC out of range"
        if stype == 0x9:
            etypestr = "Power Unit"
            if eoff == 0x0:
                especstr = "Power Down"
            if eoff == 0x1:
                especstr = "Power Cycle"
            if eoff == 0x2:
                especstr = "240VA Power Down"
            if eoff == 0x3:
                especstr = "Interlock Power Down"
            if eoff == 0x4:
                especstr = "AC Lost"
            if eoff == 0x5:
                especstr = "Soft Power Control Failure"
            if eoff == 0x6:
                especstr = "Power Unit Failure"
            if eoff == 0x7:
                especstr = "Predictive Failure"
        if stype == 0xa:
            etypestr = "Cooling Device"
        if stype == 0xb:
            etypestr = "Other Unit-based Sensor"
        if stype == 0xc:
            etypestr = "Memory"
            # spec says data3 might have bank info
            if data3:
                bank = " bank %d" % data3
            else:
                bank == ''
            if eoff == 0x0:
                especstr = "Correctable ECC error%s" % bank
            elif eoff == 0x1:
                especstr = "Uncorrectable ECC error%s" % bank
            elif eoff == 0x2:
                especstr = "Parity error%s" % bank
            elif eoff == 0x3:
                especstr = "Scrub failed%s" % bank
            elif eoff == 0x4:
                especstr = "Disabled%s" % bank
            elif eoff == 0x5:
                especstr = "ECC log limit reached%s" % bank
        if stype == 0xd:
            etypestr = "Drive Bay"
            # The bay number is encoded somewhere but I only have
            # a one-bay machine (bay 0).
            if data3 != None:
                drive = ord(data3)
            else:
                drive = ''
            if eoff == 0x0:
                especstr = "(HP) Hotswap drive %s inserted" % drive
            if eoff == 0x1:
                especstr = "(HP) Hotswap drive %s removed" % drive
        if stype == 0xe:
            etypestr = "POST Memory"
        if stype == 0xf:
            etypestr = "Boot Progress"
            if eoff == 0x0:
                especstr = "Firmware Error"
                if data2 == 0x0:
                    especstr = "Unspecified"
                if data2 == 0x1:
                    especstr = "No memory installed in system"
                if data2 == 0x2:
                    especstr = "All system memory failed"
                if data2 == 0x3:
                    especstr = "Unrecoverable ATA device failure"
                if data2 == 0x4:
                    especstr = "Unrecoverable system board failure"
                if data2 == 0x5:
                    especstr = "Unrecoverable diskette drive failure"
                if data2 == 0x6:
                    especstr = "Unrecoverable hard disk controller failure"
                if data2 == 0x7:
                    especstr = "Unrecoverable input device failure"
                if data2 == 0x8:
                    especstr = "Removable boot media not found"
                if data2 == 0x9:
                    especstr = "Unrecoverable video board failure"
                if data2 == 0xa:
                    especstr = "No video device detected"
                if data2 == 0xb:
                    especstr = "Firmware ROM corrupted"
                if data2 == 0xc:
                    epsecstr = "CPU voltage mismatch"
                if data2 == 0xd:
                    especstr = "CPU speed mismatch"
            if eoff == 0x1:
                especstr = "System Firmware Hang"
            if eoff == 0x2:
                especstr = "Firmware Progress (not decoded)"
            if eoff == 0x4:
                if data3 == 0x0:
                    especstr = "(HP) System cache size is smaller"
                if data3 == 0x1:
                    especstr = "(HP) CD-ROM not responding"
                if data3 == 0x3:
                    especstr = "(HP) Keyboard not connected"
                if data3 == 0x6:
                    especstr = "(HP) Mouse not detected"
                if data3 == 0x11:
                    especstr = "(HP) Date and time lost"
                if data3 == 0x12:
                    especstr = "(HP) Incorrect system configuration"
        if stype == 0x10:
            etypestr = "SEL Disabled"
            if eoff == 0x2:
                especstr = "System Event Log cleared"
            if eoff == 0x3:
                especstr = "All Event Logging disabled"
        if stype == 0x11:
            etypestr = "Watchdog 1"
            if eoff == 0x1:
                especstr = "OS Watchdog reset"
        if stype == 0x12:
            etypestr = "System Event"
            if eoff == 0x0:
                especstr = "System Reconfigured"
            if eoff == 0x1:
                # especstr = "OEM System Boot Event"
                # In practice, it's more like this
                especstr = "POST Completed"
            if eoff == 0x2:
                especstr = "Undetermined System Hardware Failure"
            if eoff == 0x3:
                especstr = "Entry Added to Auxiliary Log"
                # data2 has more info
            if eoff == 0x4:
                especstr = "PEF Action"
                # data2 has more info
        if stype == 0x13:
            etypestr = "Critical Interrupt"
            if eoff == 0x0:
                especstr = "Front Panel NMI"
            if eoff == 0x1:
                especstr = "Bus Timeout"
            if eoff == 0x2:
                especstr = "I/O Channel NMI"
            if eoff == 0x3:
                especstr = "Software NMI"
            if eoff == 0x4:
                especstr = "PCI PERR"
            if eoff == 0x5:
                especstr = "PCI SERR"
            if eoff == 0x6:
                especstr = "EISA Failsafe Timeout"
            if eoff == 0x7:
                especstr = "Bus Correctable Error"
            if eoff == 0x8:
                especstr = "Bus Uncorrectable Error"
            if eoff == 0x9:
                especstr = "Fatal NMI"
        if stype == 0x14:
            etypestr = "Button"
            if eoff == 0x0:
                especstr = "Power Button pressed"
            if eoff == 0x1:
                especstr = "Sleep Button pressed"
            if eoff == 0x2:
                especstr = "Reset Button pressed"
        if stype == 0x15:
            etypestr = "Module/Board"
        if stype == 0x16:
            etypestr = "Microcontroller"
        if stype == 0x17:
            etypestr = "Add-in Card"
        if stype == 0x18:
            etypestr = "Chassis"
        if stype == 0x19:
            etypestr = "Chipset"
        if stype == 0x1a:
            etypestr = "Other FRU"
        if stype == 0x1b:
            etypestr = "Interconnect"
        if stype == 0x1c:
            etypestr = "Terminator"
        if stype == 0x1d:
            etypestr = "System Boot"
            if eoff == 0x0:
                especstr = "Initiated by Power-up"
            if eoff == 0x1:
                especstr = "Initiated by Hard Reset"
            if eoff == 0x2:
                especstr = "Inititaed by Warm Reset"
            if eoff == 0x3:
                especstr = "User requested PXE boot"
            if eoff == 0x4:
                especstr = "Automatic boot to diagnostic"
        if stype == 0x1e:
            etypestr = "Boot Error"
            if eoff == 0x0:
                especstr = "No bootable media"
            if eoff == 0x1:
                especstr = "Non-bootable diskette in drive"
            if eoff == 0x2:
                especstr = "PXE server not found"
            if eoff == 0x3:
                especstr = "Invalid boot sector"
            if eoff == 0x4:
                especstr = "User boot source selection timeout"
        if stype == 0x1f:
            etypestr = "OS Boot"
            if eoff == 0x0:
                especstr = "Floppy boot completed"
            if eoff == 0x1:
                especstr = "Hard disk boot completed"
            if eoff == 0x2:
                especstr = "PXE boot completed"
            if eoff == 0x3:
                especstr = "Diagnostic boot completed"
            if eoff == 0x4:
                especstr = "CD-ROM boot completed"
            if eoff == 0x5:
                especstr = "ROM boot completed"
            if eoff == 0x6:
                especstr = "Boot completed"
        if stype == 0x20:
            etypestr = "OS Critical Stop"
            if eoff == 0x0:
                especstr = "Stop during OS initialization"
            if eoff == 0x1:
                especstr = "Runtime Stop"
        if stype == 0x21:
            etypestr = "Slot/Connector"
            if eoff == 0x0:
                especstr = "Fault"
            if eoff == 0x1:
                especstr = "Identify"
            if eoff == 0x2:
                especstr = "Device Installed"
            if eoff == 0x3:
                especstr = "Ready to Install"
            if eoff == 0x4:
                especstr = "Ready to Remove"
            if eoff == 0x5:
                especstr = "Power Off"
            if eoff == 0x6:
                especstr = "Remove Request"
            if eoff == 0x7:
                especstr = "Interlock"
            if eoff == 0x8:
                especstr = "Disabled"
            if data2 != None:
                if data2 == 0x0:
                    especstr += " - PCI"
                if data2 == 0x1:
                    especstr += " - Drive Array"
                if data2 == 0x2:
                    especstr += " - External Peripheral"
                if data2 == 0x3:
                    especstr += " - Docking"
                if data2 == 0x4:
                    especstr += " - Other Internal"
                if data2 != 0xf:
                    especstr += " - Slot %d" % data3
        if stype == 0x22:
            etypestr = "ACPI Power State"
            if eoff == 0x0:
                especstr = "S0/G0"
            if eoff == 0x1:
                especstr = "S1"
            if eoff == 0x2:
                especstr = "S2"
            if eoff == 0x3:
                especstr = "S3"
            if eoff == 0x4:
                especstr = "S4"
            if eoff == 0x5:
                especstr = "S5/G2"
            if eoff == 0x6:
                especstr = "S4/G5"
            if eoff == 0x7:
                especstr = "G3"
            if eoff == 0x8:
                especstr = "S1/S2/S3 Sleep"
            if eoff == 0x9:
                especstr = "G1 sleep"
            if eoff == 0xa:
                especstr = "S5 entered by override"
            if eoff == 0xb:
                especstr = "Legacy ON"
            if eoff == 0xc:
                especstr = "Legacy OFF"
            if eoff == 0xe:
                especstr = "Unknown"
        if stype == 0x23:
            etypestr = "Watchdog 2"
        if stype == 0x24:
            etypestr = "Platform Alert"
        if stype == 0x25:
            etypestr = "Entity Presence"
            if eoff == 0x0:
                especstr = "Present"
            if eoff == 0x1:
                especstr = "Absent"
            if eoff == 0x2:
                especstr = "Disabled"
        if stype == 0x26:
            etypestr = "Monitor ASIC"
        if stype == 0x27:
            etypestr = "LAN"
            if eoff == 0x0:
                especstr = "LAN Heartbeat lost"
            if eoff == 0x1:
                especstr = "LAN Heartbeat"
        if stype == 0x28:
            etypestr = "Management Health"
            if eoff == 0x0:
                especstr = "Sensor access degraded or unavailable"
            if eoff == 0x1:
                especstr = "Controller access degraded or unavailable"
            if eoff == 0x2:
                especstr = "Management controller offline"
            if eoff == 0x3:
                especstr = "Management controller unavailable"
        if stype == 0x29:
            etypestr = "Battery"
            if eoff == 0x0:
                especstr = "Battery low (predictive failure)"
            if eoff == 0x1:
                especstr = "Battery failed"
            if eoff == 0x2:
                especstr = "Present"
        if stype >= 0xc0 and stype <= 0xff:
            etypestr = "OEM"
        if stype == 0xc7:
            if eoff == 0x0:
                especstr = "(HP) Managment Controller started"
            if eoff == 0x1:
                especstr = "(HP) SDR area cleared"
        if stype == 0xc8:
            if eoff == 0x0:
                especstr = "(HP) %d failed Remote Management logins" % data3
            if eoff == 0x1:
                especstr = "(HP) Remote User %d Logon" % data3
            if eoff == 0x2:
                especstr = "(HP) Remote User %d Logoff" % data3
            if eoff == 0x3:
                especstr = "(HP) Remote User %d Idle Logoff" % data3
            if eoff == 0x6:
                especstr = "(HP) Remote Access configuration reset"
        if stype == 0xc9:
            if eoff == 0x3:
                especstr = "(HP) Remote Power Down"
            if eoff == 0x4:
                especstr = "(HP) Remote Power Up"
            if eoff == 0x6:
                especstr = "(HP) Front Panel Power Off"
        if stype == 0xd1:
            if eoff == 0x1:
                especstr = "(HP) Front Panel Controller Firmware updated"
            if eoff == 0x2:
                especstr = "(HP) Primary Hot Swap Controller Firmware updated"
            if eoff == 0x3:
                especstr = "(HP) Secondary Hot Swap Controller Firmware updated"
            if eoff == 0x5:
                especstr = "(HP) Management Controller Firmware updated"
            if eoff == 0x6:
                especstr = "(HP) CPU Management Controller Firmware updated"
        
    if etype >= 0x70 and etype <= 0x7f:
                etypestr = "OEM"

    return (etypestr, especstr)
