#
# Copyright (c) 2002 Doug White
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: ipmikcs.py,v 1.2 2002/05/14 23:07:03 dwhite Exp $
#
# ipmikcs.py: KCS interface for IPMI systems.
#
# Doug White

import sys
sys.path.append('ioport')
import time
from ioport import *

# Some constants ...
KCS_BASE_IOPORT	= 0xCA2
KCS_DATAIN_IOPORT =	(KCS_BASE_IOPORT)
KCS_DATAOUT_IOPORT =	(KCS_BASE_IOPORT)
KCS_STATUS_IOPORT =	(KCS_BASE_IOPORT+1)
KCS_CMD_IOPORT =	(KCS_BASE_IOPORT+1)

KCS_S1 =       	0x80
KCS_S0 =       	0x40
KCS_STATBITS =  (KCS_S0 | KCS_S1)
KCS_OEM2 =	0x20
KCS_OEM1 =	0x10
KCS_CD =       	0x08 
KCS_SMS_ATN =	0x04
KCS_IBF	=	0x02
KCS_OBF	=	0x01

KCS_IDLE =	0x0
KCS_READ =	(KCS_S0)
KCS_WRITE =	(KCS_S1)
KCS_ERROR =	(KCS_S0 & KCS_S1)

KCS_CMD_GET_STATUS =	0x60
KCS_CMD_ABORT =		0x60
KCS_CMD_WRITE_START =	0x61
KCS_CMD_WRITE_END =	0x62
KCS_CMD_READ =		0x68

KCS_SLEEP_INTERVAL = 0

def kcs_get_state():
    return inb(KCS_STATUS_IOPORT)

def kcs_print_state(status):
    print "Current KCS state: 0x%x" % status,
    if (status & KCS_STATBITS) == KCS_IDLE:
        print "IDLE_STATE ",
    elif (status & KCS_STATBITS) == KCS_READ:
        print "READ_STATE ",
    elif (status & KCS_STATBITS) == KCS_WRITE:
        print "WRITE_STATE ",
    elif (status & KCS_STATBITS) == KCS_ERROR:
        print "ERROR_STATE ",
    else:
        print "Huh? ",

    if status & KCS_IBF:
        print "IBF ",
    if status & KCS_OBF:
        print "OBF ",
    if status & KCS_OEM1:
        print "OEM1 ",
    if status & KCS_OEM2:
        print "OEM2 ",
    print ''

def kcs_wait_for_ibf_clear():
    while kcs_get_state() & KCS_IBF:
        time.sleep(KCS_SLEEP_INTERVAL)

def kcs_wait_for_obf_set():
    while not kcs_get_state() & KCS_OBF:
        time.sleep(KCS_SLEEP_INTERVAL)

def kcs_read_byte():
    return inb(KCS_DATAOUT_IOPORT)

def kcs_read_next():
    outb(KCS_DATAIN_IOPORT, KCS_CMD_READ)

def kcs_start_write():
    outb(KCS_CMD_IOPORT, KCS_CMD_WRITE_START)

def kcs_write_byte(byte):
    outb(KCS_DATAIN_IOPORT, ord(byte))

def kcs_end_write():
    outb(KCS_CMD_IOPORT, KCS_CMD_WRITE_END)

def kcs_abort():
    outb(KCS_CMD_IOPORT, KCS_CMD_ABORT)

def kcs_test_if_state(state):
    if (kcs_get_state() & KCS_STATBITS) == (KCS_STATBITS & state):
        return 1
    else:
        return 0

def kcs_clear_obf():
    if kcs_get_state() & KCS_OBF:
       kcs_read_byte()

def kcs_read():
    data = ""

    kcs_wait_for_ibf_clear()
    if not kcs_test_if_state(KCS_READ):
        print "Not in READ state?"
        return None

    while kcs_test_if_state(KCS_READ):
        kcs_wait_for_obf_set()
        data = data + chr(kcs_read_byte())
        kcs_read_next()
        kcs_wait_for_ibf_clear()

    if kcs_test_if_state(KCS_IDLE):
        kcs_clear_obf()
        return data
    else:
        return None

def kcs_write(data):

    l = 0
    
    kcs_wait_for_ibf_clear()
    kcs_clear_obf()
    kcs_start_write()
    kcs_wait_for_ibf_clear()
    if not kcs_test_if_state(KCS_WRITE):
        print "Not in WRITE state?"
        return None

    kcs_clear_obf()
    for x in data[:-1]:
        kcs_write_byte(x)
        l = l + 1
        kcs_wait_for_ibf_clear()
        if not kcs_test_if_state(KCS_WRITE):
            print "Got out of WRITE state in write loop"
            return None
        kcs_clear_obf()

    kcs_end_write()
    kcs_wait_for_ibf_clear()
    if not kcs_test_if_state(KCS_WRITE):
        print "Got out of write state when done"
        return None
    kcs_clear_obf()
    kcs_write_byte(data[-1])
    l = l + 1

    return l


    
