#
# Copyright (c) 2002 Doug White
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: ipmisensor.py,v 1.8 2002/07/19 00:09:18 dwhite Exp $
#
# ipmisensor.py
#

import struct
import ipmiexception
import ipmimessage

def get_sensor_reading(sess, ownid, olun, sensor):
    # ret = sess.send(ownid, olun, 0x81, 0x0, 1, 0x4, 0x2D, struct.pack('<B', int(sensor)))
    mess = ipmimessage.IPMIMessage()
    mess.quickload(ownid, olun, 0x20, 0x0, sess.next_seq(), 0x4, 0x2D, struct.pack('<B', int(sensor)))
    # print "get_sensor_reading: owner: 0x%x  sensor: 0x%x" % (ownid, sensor)
    rm = sess.sendmessage(mess)
    ret = rm.data

    # only thresh-type sensors for now

##    if ord(ret[0]) != 0x0:
##        raise ValueError, "Get Sensor Reading error (0x%x), sensor 0x%x" % (ord(ret[0]), sensor)
    
    ccode, reading, evtflag, thresh = struct.unpack('<4B', ret[:4])

    if ccode != 0x0:
        raise ValueError, "Get Sensor Reading error (%d)" % ccode

    return (reading, evtflag, thresh)


def get_sensor_status(sess, ownid, olun, sensor):
    # ret = sess.send(ownid, olun, 0x81, 0x0, 1, 0x4, 0x2B, struct.pack('<B', int(sensor)))
    mess = ipmimessage.IPMIMessage()
    mess.quickload(ownid, olun, 0x20, 0x2, sess.next_seq(), 0x4, 0x2B, struct.pack('<B', int(sensor)))
    # print "get_sensor_status: owner: 0x%x  sensor: 0x%x" % (ownid, sensor)
    rm = sess.sendmessage(mess)
    ret = rm.data
        
    fstr = "<%dB" % len(ret)
##    if ord(ret[0]) != 0x0:
##        raise ValueError, "Get Sensor Event Status error (0x%x), sensor 0x%x" % (ord(ret[0]), sensor)
    try:
        ccode, updates, thresho, thresht, threshu, threshv = struct.unpack(fstr, ret)
    except:
        print "fstr = %s" % fstr
        print "len = %s" % len(ret)
        print "sensor = 0x%x" % sensor
        raise

    return (updates, thresho, thresht, threshu, threshv)


def convert_raw(reading, sdr):
    # To print a sensor, we need to fill in this equation:
    #
    # y = Mx + (B * 10^Bexp) * 10^Rexp

    # y = Linearize(y) if the Linearization enum is set.

    # sdr should be the SDR entry for the given reading.

    bexp = int(sdr['b-exponent'])
    rexp = int(sdr['r-exponent'])

    out = (sdr['m'] * reading + (sdr['b'] * (10.0**bexp))) * 10.0**rexp

    #print "Converted reading: %s" % out

    unit = ""

    if sdr['modifier-unit-type'] == 1:
        unit = "%s/%s" % (sdr['base-unit'], sdr['modifier-unit'])
    elif sdr['modifier-unit-type'] == 2:
        unit = "%s * %s" % (sdr['base-unit'], sdr['modifier-unit'])
    else:
        unit = sdr['base-unit']

    return "%s %s" % (out, unit)


def decode_read_thresh(thresh):
    print "%x" % thresh
    if thresh & 0x20:
        return 'Non-Recoverable'
    elif thresh & 0x10:
        return 'Critical'
    elif thresh & 0x8:
        return 'Non-Critical'
    elif thresh & 0x4:
        return 'Non-Recoverable'
    elif thresh & 0x2:
        return 'Critical'
    elif thresh & 0x1:
        return 'Non-Critical'
    else:
        return 'Normal'


def decode_status_thresh(thresho, thresht):
    # print "Thresh1: %x  Thresh2: %x" % (thresho, thresht)
    if thresho & 0x80:
        return ('Upper', 'Non-Critical', 'High')
    elif thresho & 0x40:
        return ('Upper', 'Non-Critical', 'Low')
    elif thresho & 0x20:
        return ('Lower', 'Non-Recoverable', 'High')
    elif thresho & 0x10:
        return ('Lower', 'Non-Recoverable', 'Low')
    elif thresho & 0x8:
        return ('Lower', 'Critical', 'High')
    elif thresho & 0x4:
        return ('Lower', 'Critical', 'Low')
    elif thresho & 0x2:
        return ('Lower', 'Non-Critical', 'High')
    elif thresho & 0x1:
        return ('Lower', 'Non-Critical', 'Low')

    if thresht & 0x8:
        return ('Upper', 'Non-Recoverable', 'High')
    elif thresht & 0x4:
        return ('Upper', 'Non-Recoverable', 'Low')
    elif thresht & 0x2:
        return ('Upper', 'Critical', 'High')
    elif thresht & 0x1:
        return ('Upper', 'Critical', 'Low')
    else:
        return ("Normal", "Normal", "Normal")


def get_and_compare_sensor(sess, ownid, olun, sensor, sdr):

    # Get current sensor trigger status
    try:
        threshdata = get_sensor_status(sess, ownid, olun, sensor)
    except ipmiexception.IPMICommandError, e:
        if e.value == 0xcb:
            # no thresh info
            threshdata = None
        else:
            raise

    # may not be a threshold sensor...
    if threshdata:
        #if threshdata[0] & 0x40 == 0:
            # scanning disabled
        #    raise ipmiexception.IPMISensorDisabled
        # otherwise decode the threshold bits
        crit = decode_status_thresh(threshdata[1], threshdata[2])

    # Now try to get the actual reading.
    try:
        sensdata = get_sensor_reading(sess, ownid, olun, sensor)
    except ipmiexception.IPMICommandError, e:
        if e.value == 0xcb:
            raise ipmiexception.IPMISensorDisabled
        else:
            raise
    # Convert units.
    val = convert_raw(sensdata[0], sdr)

    return (val, crit)
