#
# Copyright (c) 2002 Doug White
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: ipmitransport.py,v 1.5 2002/07/19 03:30:40 dwhite Exp $
#
# ipmitransport.py -- common data transport routines
#

import struct
import ipmiexception
import ipmimessage

# load_chunked_data: Read data records using netfn/cmd
def load_chunked_record(sess, daddr, dlun, saddr, slun, sseq, netfn, cmd, resvid, recnum, reclen):
    # resvid is our reservation id.
    if resvid == None:
        resvid = 0

    #ret = sess.send(daddr, dlun, saddr, slun, sseq, netfn, cmd, struct.pack('<6B', resvid & 0x00ff, (resvid & 0xff00) >> 8, recnum & 0x00ff, (recnum & 0xff00) >> 8, 0x0, 0x5))
    m = ipmimessage.IPMIMessage()
    m.quickload(daddr, dlun, saddr, slun, sseq, netfn, cmd, struct.pack('<6B', resvid & 0x00ff, (resvid & 0xff00) >> 8, recnum & 0x00ff, (recnum & 0xff00) >> 8, 0x0, 0x5))
    rm = sess.sendmessage(m)
    ret = rm.data
    #pbytes(ret)
    # XXX needs exception handling intead of explicit check
    if ord(ret[0]) == 0x81 or ord(ret[0]) == 0xC5:
        raise ValueError, "Reservation cancelled (0x%x)" % ord(ret[0])
    elif ord(ret[0]) != 0x0:
        raise ValueError, "Error occured getting record (0x%x)" % ord(ret[0])

    # Parse out header.
    # rlen is the number of bytes yet to grab
    # If the caller gives a non-zero reclen, use that
    if reclen == 0 or reclen == None:
        ridlo, ridhi, sdrver, rtype, rlen = struct.unpack('<BBBBB', ret[3:8])
    else:
        rlen = reclen - 5
    #print "#%d: Type 0x%x, len %d" % ((ridhi << 8)+ridlo, rtype, rlen)

    chunkoff = 5
    chunklen = 30
    # Grab the rest of the bytes.
    while rlen > 0:
        # Grab 30 byte chunks until len is depleted
        # Looks like the 'bytes to read too big' thing isn't portable.
        if rlen < chunklen:
            chunklen = rlen

        # need to rip off completion code & header info again
        #print ">> Getting chunk, offset = %d, len = %d" % (chunkoff, chunklen)
        try:
            # cbody = sess.send(daddr, dlun, saddr, slun, sseq, netfn, cmd, struct.pack('<6B', resvid & 0x00ff, (resvid & 0xff00) >> 8, recnum & 0x00ff, (recnum & 0xff00) >> 8, chunkoff, chunklen))
            cm = ipmimessage.IPMIMessage()
            cm.quickload(daddr, dlun, saddr, slun, sseq, netfn, cmd, struct.pack('<6B', resvid & 0x00ff, (resvid & 0xff00) >> 8, recnum & 0x00ff, (recnum & 0xff00) >> 8, chunkoff, chunklen))
            rm = sess.sendmessage(cm)
            cbody = rm.data
        except ipmiexception.IPMICommandError, e:
            if e.value == 0xc9 and chunklen != 16:
                # "Parameter out of range"
                # on the HPs, they can only handle a max chunksize
                # of 16 bytes, while the Intels can do better
                # than 30. Let's try resetting the chunksize
                # and giving it another spin.
                #print "**** Trying chunklen = 16"
                chunklen = 16
                continue
            else:
                # print "******* Error getting chunk at len %d (0x%x)" % (chunkoff, ord(cbody[0]))
                #pbytes(cbody)
                raise ValueError, "Error getting chunk at len %d (0x%x)" % (chunkoff, ord(cbody[0]))
        #print ">> Got %d bytes in last chunk" % len(cbody[3:])
        ret = ret + cbody[3:]
        rlen = rlen - chunklen
        chunkoff = chunkoff + chunklen                   


    return ret

# A simpler chunk grabber for FRU records, which have a free-floating format
# Just keep grabbing bytes until you hit reclen

def load_fru_chunked_record(sess, saddr, slun, sseq, netfn, cmd, fsdr, off, reclen):
    chunklen = 30
    chunkoff = off
    ret = ""
    if reclen < chunklen:
        chunklen = reclen
    
    while len(ret) < reclen:
        loc = struct.pack('<4B', fsdr['device-address'], chunkoff, 0, chunklen)
        try:
            cbody = sess.send(fsdr['device-access'], fsdr['device-lun'], saddr, slun, sseq, netfn, cmd, loc)
            m = ipmimessage.IPMIMessage()
            m.quickload(fsdr['device-access'], fsdr['device-lun'], saddr, slun, sseq, netfn, cmd, loc)
            rm = sess.sendmessage(m)
            cbody = rm.data
        except ipmiexception.IPMICommandError, err:
            if (err.value == 0xc8 or err.value == 0xc9) and chunklen != 16:
                # HPs overflowed, reduce size & try again
                chunklen = 16
                continue
            else:
                raise 
        ret = ret + cbody[2:]
        chunkoff = chunkoff + chunklen
        if reclen - len(ret)  > chunklen:
            chunklen = reclen - len(ret)

    return ret
