/*-
 * Copyright (c) 2002 Doug White
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * 
 */

/*
 *  smic-send.c
 *  
 */
 
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include "smic.h"
#include "smic-common.h"
#include "kcs-common.h"
#include "ipmi-common.h"

void
usage()
{
    printf("You fucked up\n");
    exit(1);
}

int
main(int argc, char** argv)
{
    int t,u;
    
    int command=0;
    int cmdlen=0;
    int lun=0;
    int netfn=0;
    int verbose=0;
    int ret=0;
    u_int8_t cmd[255];
    u_int8_t out[255];

    while((u = getopt(argc, argv, "c:l:n:v")) != -1)
    {
            switch(u) {
            case 'c':
                    command=strtol(optarg, NULL, 0);
                    break;
            case 'l':
                    lun=strtol(optarg, NULL, 0);
                    break;
            case 'n':
                    netfn=strtol(optarg, NULL, 0);
                    break;
            case 'v':
                    verbose=1;
                    break;
            case '?':
            default:
                    usage();
            }
    }

    argc -= optind;
    argv += optind;
    
    if(command == 0 || netfn == 0) {
		usage();
    }
    if(verbose) {
        printf("Setting NetFn %#x, Command %#x\n", netfn, command);
    }

    t = i386_set_ioperm(SMIC_BASE_PORT, 0x3, 0x1);
    if(t != 0) {
	perror("i386_set_ioperm");
	exit(1);
    }

    cmd[0] = ipmi_compose_netfn_lun(netfn, lun);
    cmd[1] = command;

    cmdlen = 2;
    
    for(u=0; u < argc; u++) {
        if(verbose) {
            printf("Adding data byte #%d: \"%s\" %#x\n", u, argv[u], argv[u]);
        }
        cmd[cmdlen] = strtol(argv[u], NULL, 0);
        cmdlen++;
	
    }
    if(verbose) {
        printf("Sending %d byte command\n", cmdlen);
    }

    if(smic_write(cmd, cmdlen) == -1)
    {
        errx(1, "Error writing command\n");
    }
    ret = smic_read(out, 255);
    if(ret < 0)
    {
        errx(2, "Error reading response\n");
    }
    
    printf("LUN            : %#0.2x\n", out[0] & 0x3);
    printf("NetFn          : %#0.2x\n", out[0] >> 2);
    printf("Command        : %#0.2x\n", out[1]);
    printf("Completion Code: %#0.2x\n", (out[2] & 0xff));

    kcs_decode_packet(ret, out);
    
    if(verbose) {
        for(u=0; u<ret; u++) {
            if(u%8 == 0) printf("\n");
            printf("0x%x ", out[u]);
        }
		
        printf("\n");
    }

}

